<?php  if ( ! defined('BASEPATH')) exit('No direct script access allowed');
/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" :
 * <thepixeldeveloper@googlemail.com> wrote this file. As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return Mathew Davies
 * ----------------------------------------------------------------------------
 */
 
/**
* redux_auth_model
*/
class redux_auth_model extends Model
{
	/**
	 * Holds an array of tables used in
	 * redux.
	 *
	 * @var string
	 **/
	public $tables = array();
	
	/**
	 * activation code
	 *
	 * @var string
	 **/
	public $activation_code;
	
	/**
	 * forgotten password key
	 *
	 * @var string
	 **/
	public $forgotten_password_code;
	
	/**
	 * new password
	 *
	 * @var string
	 **/
	public $new_password;
	
	/** array */
	public $profile = array();
	
	public $network = array();
	public $groups_network = array();
	public $friends_network = array();
	public $experts_network = array();
	
	/**
	 * Identity
	 *
	 * @var string
	 **/
	public $identity;
	
	public function __construct()
	{
		parent::__construct();
		$this->tables  = $this->config->item('tables');
		$this->columns = $this->config->item('columns');
		$this->col_points = $this->config->item('col_points');
	}
	
	/**
	 * Misc functions
	 * 
	 * Hash password : Hashes the password to be stored in the database.
     * Hash password db : This function takes a password and validates it
     * against an entry in the users table.
     * Salt : Generates a random salt value.
	 *
	 * @author Mathew
	 */
	 
	/**
	 * Hashes the password to be stored in the database.
	 *
	 * @return void
	 * @author Mathew
	 **/
	public function hash_password($password = false)
	{
	    $salt_length = $this->config->item('salt_length');
	    
	    if ($password === false)
	    {
	        return false;
	    }
	    
		$salt = $this->salt();
		
		$password = $salt . substr(sha1($salt . $password), 0, -$salt_length);
		
		return $password;		
	}
	
	/**
	 * This function takes a password and validates it
     * against an entry in the users table.
	 *
	 * @return void
	 * @author Mathew
	 **/
	public function hash_password_db($identity = false, $password = false)
	{
	    $identity_column   = $this->config->item('identity');
	    $users_table       = $this->tables['users'];
	    $salt_length       = $this->config->item('salt_length');
	    
	    if ($identity === false || $password === false)
	    {
	        return false;
	    }
	    
	    $query  = $this->db->select('password')
                    	   ->where($identity_column, $identity)
                    	   ->limit(1)
                    	   ->get($users_table);
            
        $result = $query->row();
        
		if ($query->num_rows() !== 1)
		{
		    return false;
	    }
	    
		$salt = substr($result->password, 0, $salt_length);

		$password = $salt . substr(sha1($salt . $password), 0, -$salt_length);
        
		return $password;
	}
	
	/**
	 * Generates a random salt value.
	 *
	 * @return void
	 * @author Mathew
	 **/
	public function salt()
	{
		return substr(md5(uniqid(rand(), true)), 0, $this->config->item('salt_length'));
	}
    
	/**
	 * Activation functions
	 * 
     * Activate : Validates and removes activation code.
     * Deactivae : Updates a users row with an activation code.
	 *
	 * @author Mathew
	 */
	
	/**
	 * activate
	 *
	 * @return void
	 * @author Mathew
	 **/
	public function activate($code = false,$id = false)
	{
	    $identity_column = $this->config->item('identity');
	    $users_table     = $this->tables['users'];
	    
	    if (($code === false)||($id===false))
	    {
	        return false;
	    }
	  
	    $query = $this->db->select($identity_column)
                	      ->where('activation_code', $code)
                	      //->where('')
                	      ->limit(1)
                	      ->get($users_table);
                	      
		$result = $query->row();
        
		if ($query->num_rows() !== 1)
		{
		    
        return false;
		}
	    
		$identity = $result->{$identity_column};
		
		if ($identity != $id){
      return false;
    }
		
		$data = array('activation_code' => '0');
        
		$this->db->update($users_table, $data, array($identity_column => $identity));
		
		return ($this->db->affected_rows() == 1) ? true : false;
	}
	
	/**
	 * Deactivate
	 *
	 * @return void
	 * @author Mathew
	 **/
	public function deactivate($id = false)
	{
	    $users_table = $this->tables['users'];
	    $identity_column = $this->config->item('identity');
	    if ($id === false)
	    {
	        return false;
	    }
	    
		$activation_code = sha1(md5(microtime()));
		$this->activation_code = $activation_code;
		
		$data = array('activation_code' => $activation_code);
        
		$this->db->update($users_table, $data, array($identity_column => $id));
		
		return ($this->db->affected_rows() == 1) ? true : false;
	}

	/**
	 * change password
	 *
	 * @return void
	 * @author Mathew
	 **/
	public function change_password($identity = false, $old = false, $new = false)
	{
	    $identity_column   = $this->config->item('identity');
	    $users_table       = $this->tables['users'];
	    
	    if ($identity === false || $old === false || $new === false)
	    {
	        return false;
	    }
	    
	    $query  = $this->db->select('password')
                    	   ->where($identity_column, $identity)
                    	   ->limit(1)
                    	   ->get($users_table);
                    	   
	    $result = $query->row();

	    $db_password = $result->password; 
	    $old         = $this->hash_password_db($identity, $old);
	    $new         = $this->hash_password($new);

	    if ($db_password === $old)
	    {
	        $data = array('password' => $new);
	        
	        $this->db->update($users_table, $data, array($identity_column => $identity));
	        
	        return ($this->db->affected_rows() == 1) ? true : false;
	    }
	    
	    return false;
	}
	
	/**
	 * Checks username.
	 *
	 * @return void
	 * @author Mathew
	 **/
	public function username_check($username = false)
	{
	    $users_table = $this->tables['users'];
	    
	    if ($username === false)
	    {
	        return false;
	    }
	    
	    $query = $this->db->select('users_id')
                           ->where('username', $username)
                           ->limit(1)
                           ->get($users_table);
		
		if ($query->num_rows() == 1)
		{
			return true;
		}
		
		return false;
	}
	
	/**
	 * Checks email.
	 *
	 * @return void
	 * @author Mathew
	 **/
	public function email_check($email = false)
	{
	    $users_table = $this->tables['users'];
	    
	    if ($email === false)
	    {
	        return false;
	    }
	    
	    $query = $this->db->select('users_id')
                           ->where('email', $email)
                           ->limit(1)
                           ->get($users_table);
		
		if ($query->num_rows() == 1)
		{
			return true;
		}
		
		return false;
	}
	
	/**
	 * Identity check
	 *
	 * @return void
	 * @author Mathew
	 **/
	protected function identity_check($identity = false)
	{
	    $identity_column = $this->config->item('identity');
	    $users_table     = $this->tables['users'];
	    
	    if ($identity === false)
	    {
	        return false;
	    }
	    
	    $query = $this->db->select('users_id')
                           ->where($identity_column, $identity)
                           ->limit(1)
                           ->get($users_table);
		
		if ($query->num_rows() == 1)
		{
			return true;
		}
		
		return false;
	}

	/**
	 * Insert a forgotten password key.
	 *
	 * @return void
	 * @author Mathew
	 **/
	public function forgotten_password($email = false)
	{
	    $users_table = $this->tables['users'];
	    
	    if ($email === false)
	    {
	        return false;
	    }
	    
	    $query = $this->db->select('forgotten_password_code')
                    	   ->where('email', $email)
                    	   ->limit(1)
                    	   ->get($users_table);
            
        $result = $query->row();
		
		$code = $result->forgotten_password_code;

		if (empty($code))
		{
			$key = $this->hash_password(microtime().$email);
			
			$this->forgotten_password_code = $key;
		
			$data = array('forgotten_password_code' => $key);
			
			$this->db->update($users_table, $data, array('email' => $email));
			
			return ($this->db->affected_rows() == 1) ? true : false;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * undocumented function
	 *
	 * @return void
	 * @author Mathew
	 **/
	public function forgotten_password_complete($code = false)
	{
	    $users_table = $this->tables['users'];
	    $identity_column = $this->config->item('identity'); 
	    
	    if ($code === false)
	    {
	        return false;
	    }
	    
	    $query = $this->db->select('users_id')
                    	   ->where('forgotten_password_code', $code)
                           ->limit(1)
                    	   ->get($users_table);
        
        $result = $query->row();
        
        if ($query->num_rows() > 0)
        {
            $salt       = $this->salt();
		    $password   = $this->hash_password($salt);
		    
		    $this->new_password = $salt;
		    
            $data = array('password'                => $password,
                          'forgotten_password_code' => '0');
            
            $this->db->update($users_table, $data, array('forgotten_password_code' => $code));

            return true;
        }
        
        return false;
	}

	/**
	 * profile
	 *
	 * @return void
	 * @author Mathew
	 **/
	public function profile($identity = false)
	{
	    $users_table     = $this->tables['users'];
	    //$groups_table    = $this->tables['groups'];
	    $users_meta_table      = $this->tables['users_meta'];
		//$points_table = $this->tables['points'];
	    $users_meta_join       = $this->config->item('join');
	    $identity_column = $this->config->item('identity');    
	    
	    if ($identity === false)
	    {
	        return false;
	    }
	    
		$this->db->select($users_table.'.users_id, '.
						  $users_table.'.full_name, ' .
						  $users_table.'.password, '.
						  $users_table.'.email, '.
						  $users_table.'.activation_code, '.
						  $users_table.'.forgotten_password_code , '.
						  $users_table.'.ip_address');

		
		if (!empty($this->columns))
		{
		    foreach ($this->columns as $value)
    		{
    			$this->db->select($users_meta_table.'.'.$value);
    		}
		}
		//$this->db->select($points_table.".points_level");
		//$this->db->select("(".$points_table.".points_multimed+".$points_table.".points_wikimed+".$points_table.".points_community+".$points_table.".points_bonus+".$points_table.".points_mediscuss+".$points_table.".points_quiz) as sum_points");

		
		$this->db->from($users_table);
		$this->db->join($users_meta_table, $users_table.'.users_id = '.$users_meta_table.'.'.$users_meta_join, 'left');
		//$this->db->join($points_table, $users_table.'.users_id ='.$points_table.'.'.$users_meta_join,'left');
		//$this->db->join($groups_table, $users_table.'.group_id = '.$groups_table.'.id', 'left');
		
		if (strlen($identity) === 40)
	    {
	        $this->db->where($users_table.'.forgotten_password_code', $identity);
	    }
	    else
	    {
	        $this->db->where($users_table.'.'.$identity_column, $identity);
	    }
	    
		$this->db->limit(1);
		$i = $this->db->get();
		
		if ($i->num_rows > 0){
			$this->profile = $i->row();
			return $this->profile;
		}else{
			false;
		}
	}
	
	public function user_meta($id,$key){
		$users_table     = $this->tables['users'];
	    //$groups_table    = $this->tables['groups'];
	    $users_meta_table      = $this->tables['meta'];
		//$points_table = $this->tables['points'];
	    $users_meta_join       = $this->config->item('join');
	    $identity_column = $this->config->item('identity');    
	    if ($identity === false)
	    {
	        return false;
	    }
	    
		
    	    			$this->db->select($users_meta_table.'.'.$value);
    	
		
		$this->db->select($points_table.".points_level");
		$this->db->select("(".$points_table.".points_multimed+".$points_table.".points_wikimed+".$points_table.".points_community+".$points_table.".points_bonus+".$points_table.".points_mediscuss+".$points_table.".points_quiz) as sum_points");

		
		$this->db->from($users_table);
		$this->db->join($users_meta_table, $users_table.'.users_id = '.$users_meta_table.'.'.$users_meta_join, 'left');
		$this->db->join($points_table, $users_table.'.users_id ='.$points_table.'.'.$users_meta_join,'left');
		//$this->db->join($groups_table, $users_table.'.group_id = '.$groups_table.'.id', 'left');
		
		if (strlen($identity) === 40)
	    {
	        $this->db->where($users_table.'.forgotten_password_code', $identity);
	    }
	    else
	    {
	        $this->db->where($users_table.'.'.$identity_column, $identity);
	    }
	    
		$this->db->limit(1);
		$i = $this->db->get();
		
		if ($i->num_rows > 0){
			$this->profile = $i->row();
			return $this->profile;
		}else{
			false;
		}
	}
	
	


	/**
	 * Basic functionality
	 * 
	 * Register
	 * Login
	 *
	 * @author Mathew
	 */
	
	/**
	 * register
	 *
	 * @return void
	 * @author Mathew
	 **/
	public function register($username = false, $password = false, $email = false)
	{
	    $users_table        = $this->tables['users'];
	    $users_meta_table         = $this->tables['users_meta'];
	    //$groups_table       = $this->tables['groups'];
	    $users_meta_join          = $this->config->item('join');
	    $additional_columns = $this->config->item('columns');
	    
	    if ($username === false || $password === false || $email === false)
	    {
	        return false;
	    }
	    
        // Group ID
	    /*$query    = $this->db->select('id')->where('name', $this->config->item('default_group'))->get($groups_table);
	    $result   = $query->row();
	    $group_id = $result->id;*/
	    
        // IP Address
      $ip_address = $this->input->ip_address();
	    $password = $this->hash_password($password);
      $register_date = mdate("%Y-%m-%d %h:%i:%a", now());
		
        // Users table.
		$data = array('full_name' => $username, 
					  'password' => $password, 
					  'email'    => $email,
					  'date_register' => $register_date,
					  'ip_address' => $ip_address);
		  
		$this->db->insert($users_table, $data);
        
		// users_meta table.
		$id = $this->db->insert_id();
		
		
		$data = array($users_meta_join => $id);
		
		/*if (!empty($additional_columns))
	    {
	        foreach ($additional_columns as $input)
	        {
	            $data[$input] = $this->input->post($input);
	        }
	    }*/
        
		$this->db->insert($users_meta_table, $data);
		return ($this->db->affected_rows() > 0) ? $id : false;
	}
	
	/**
	 * login
	 *
	 * @return void
	 * @author Mathew
	 **/
	public function login($identity = false, $password = false)
	{
	    $identity_column = $this->config->item('identity');
	    $users_table     = $this->tables['users'];
	    
	    if ($identity === false || $password === false || $this->identity_check($identity) == false)
	    {
	        return false;
	    }
	    
	    $query = $this->db->select($identity_column.', password, activation_code, users_id,username')
                    	   ->where($identity_column, $identity)
                    	   ->limit(1)
                    	   ->get($users_table);
	    
        $result = $query->row();
        
        if ($query->num_rows() == 1)
        {
            $password = $this->hash_password_db($identity, $password);
            
            //if (!empty($result->activation_code)) { return false; }
            
    		if ($result->password === $password)
    		{
    		    $session = array(
                   $identity_column  => $result->{$identity_column},
                   'users_id'     =>  $result->{"users_id"}
               );
				$this->session->set_userdata($session);

    		    return true;
    		}
        }
        
		return false;		
	}
	
		/**
	 * login
	 *
	 * @return void
	 * @author Mathew
	 **/
	public function is_activate($identity = false)
	{
	    $identity_column = $this->config->item('identity');
	    $users_table     = $this->tables['users'];
	    
	    if ($identity === false || $this->identity_check($identity) == false)
	    {
	        return false;
	    }
	    
	    $query = $this->db->select('activation_code')
                    	   ->where($identity_column, $identity)
                    	   ->limit(1)
                    	   ->get($users_table);
	    
        $result = $query->row();
        
        if ($query->num_rows() == 1)
        {
            //$password = $this->hash_password_db($identity, $password);
            
            if (!empty($result->activation_code)) { 
              return false; 
            }else{
              return true;
            }
            
    		
        }
        
		return false;		
	}
	
	public function network($user_id)
	{
		$network = array();
		$this->db->select("users_id_request");
		$this->db->where("users_id_approve",$user_id);
		$this->db->where("status","friend");
		$query = $this->db->get("friendship");
		foreach ($query->result_array() as $row)
		{
			$network[]=$row["users_id_request"];
		}
		$this->db->select("users_id_approve");
		$this->db->where("users_id_request",$user_id);
		$this->db->where("status","friend");
		$query = $this->db->get("friendship");
		foreach ($query->result_array() as $row)
		{
			$network[]=$row["users_id_approve"];
		}
		$this->db->select("users_id_approve");
		$this->db->where("users_id_request",$user_id);
		$this->db->where("status","follow");
		$query = $this->db->get("friendship");
		foreach ($query->result_array() as $row)
		{
			$network[]=$row["users_id_approve"];
		}
		$this->network = $network;
		return $this->network;
  }
	public function experts_network($user_id)
	{
		$network = array();

		$this->db->select("users_id_approve");
		$this->db->where("users_id_request",$user_id);
		$this->db->where("status","follow");
		$query = $this->db->get("friendship");
		foreach ($query->result_array() as $row)
		{
			$network[]=$row["users_id_approve"];
		}
		$this->experts_network = $network;
		return $this->experts_network;
	}
  
	public function friends_network($user_id)
	{
		$network = array();
		$this->db->select("users_id_request");
		$this->db->where("users_id_approve",$user_id);
		$this->db->where("status","friend");
		$query = $this->db->get("friendship");
		$result = $query->result_array();
		foreach($result as $item){
			$network[] = $item["users_id_request"];	
		}
		
		$this->db->select("users_id_approve");
		$this->db->where("users_id_request",$user_id);
		$this->db->where("status","friend");
		$query = $this->db->get("friendship");
		$result = $query->result_array();
		foreach($result as $item){
			$network[] = $item["users_id_approve"];	
		}
		
		$this->experts_network = $network;
		return $this->experts_network;
	}
	
	public function groups_network($users_id){
		$all_group = array();
		$this->db->select("groups.groups_id");
		$this->db->from("groups");
		$this->db->join("users","users.users_id=groups.groups_creator","left");
		$this->db->join("groups_members","groups.groups_id=groups_members.groups_id","left");
		$where = "(groups_creator=".$users_id. " OR(groups_members.users_id=".$users_id." AND (groups_members.users_status='member' OR groups_members.users_status='admin')))";
		$this->db->where($where);
		$i = $this->db->get();
		$groups_network = $i->result_array();
		foreach($groups_network as $item){
			$all_group[] = $item["groups_id"];
		}
		$this->groups_network = $all_group;
		return $this->groups_network;
	}
	
	
}
